import base64
import re
import asyncio
from playwright.async_api import async_playwright, TimeoutError
from loguru import logger

from src.core.message_client import send_message
from src.entity.account import Account, AccountType
import aiohttp

from src.entity.database import db


# 提取重复的请求和响应处理逻辑
async def handle_event(page, event_type, callback, *args):
    page.on(event_type, lambda event: asyncio.create_task(callback(event, *args)))


async def handle_request(request, result_container, request_event):
    if 'perInfo' in request.url:
        headers = request.headers
        result_container['headers'] = headers
        logger.info(f'Successfully got headers: {headers}')
        request_event.set()


async def handle_response(response, response_container, response_event):
    if 'loginFlow' in response.url:
        data = await response.json()
        response_container['data'] = data
        response_event.set()


async def playwright_login(account: Account) -> dict:
    result_container = {'headers': {}}
    response_container = {'data': {}}
    logger.info(f'Starting login for username: {account.username}')
    try:
        async with async_playwright() as playwright:
            await perform_login(playwright, account, result_container, response_container)
    except Exception as e:
        logger.error(f'Error during login for account {account.username}: {e}', exc_info=True)
        await handle_login_failure(account)
    return result_container['headers']


async def perform_login(playwright, account: Account, request_container: dict,
                        response_container: dict):
    browser = await playwright.chromium.launch(headless=False)
    context = await browser.new_context()
    page = await context.new_page()

    request_event = asyncio.Event()
    response_event = asyncio.Event()

    await page.goto(account.url)

    await handle_event(page, 'request', handle_request, request_container, request_event)
    await handle_event(page, 'response', handle_response, response_container, response_event)
    # 填充基本的元素
    await fill_form_common(page, account)
    # 如果是九游 验证码滑动,并且滑动之后自动登录成功
    if account.type == AccountType.jy:
        await handle_jy_captcha(page, account, 0)
        logger.info("验证码处理成功")
        try:
            # 等待请求发送 获取header返回值
            await request_event.wait()
        except TimeoutError:
            # 超时 即验证码处理失败
            logger.warning("登录处理失败")
    else:
        # 验证码输入成功
        max_retries = 3
        for attempt in range(max_retries):
            if await handle_captcha(page):
                response_event.clear()  # 重置事件
                await page.locator("div").filter(has_text=re.compile(r"^登录$")).get_by_role("button").click()
                try:
                    await response_event.wait()
                    if response_container['data'].get('status_code') != 6000:
                        logger.warning(f"登录发生错误, {response_container['data'].get('message', '未知错误')}")
                        close_button = page.locator("button.ant-modal-close")
                        await close_button.wait_for()
                        await close_button.click()
                    else:
                        break
                except TimeoutError:
                    logger.warning(f"等待响应超时，重试 {attempt + 1}/{max_retries}")
            else:
                logger.warning(f"验证码输入失败，重试 {attempt + 1}/{max_retries}")
        else:
            logger.warning("登录失败，达到最大重试次数")
        try:
            # 等待请求发送 获取header返回值
            await request_event.wait()
        except TimeoutError:
            # 超时 即验证码处理失败
            logger.warning("登录处理失败")
        await close_resources(page, context, browser)


async def fill_form_common(page, account: Account):
    username_input = page.get_by_placeholder('用户名')
    password_input = page.get_by_placeholder('密码')
    await username_input.click()
    await username_input.fill(account.username)
    await password_input.click()
    await password_input.fill(account.password)


async def handle_jy_captcha(page, account, retry_count=0):
    await page.locator("div").filter(has_text=re.compile(r"^登录$")).get_by_role("button").click()
    validate_code = page.locator('.geetest_box')
    await validate_code.wait_for(state='visible')
    await asyncio.sleep(1)
    validate_code_buffer = await validate_code.screenshot()
    img = base64.b64encode(validate_code_buffer).decode('utf-8')
    res = await base64_api(img=img, typeid=33)
    drag_btn = validate_code.locator(".geetest_btn")
    drag_btn_box = await drag_btn.bounding_box()
    try:
        distance = int(res) + 3
        target_x = drag_btn_box['x'] + distance
        await drag_btn.hover()
        await page.mouse.down()
        await page.mouse.move(target_x, drag_btn_box["y"], steps=25)
        await page.mouse.up()
        await validate_code.wait_for(state='hidden')
        return True
    except (ValueError, TimeoutError):
        retry_count += 1
        logger.error(f"验证码处理失败，重试: {res}")
        return await handle_jy_captcha(page, account, retry_count)


async def handle_captcha(page):
    validate_code_input = page.get_by_placeholder('请输入验证码')
    validate_code_img = page.locator('#boxModle')
    await validate_code_img.wait_for()
    validate_code_buffer = await validate_code_img.screenshot()
    img_base64 = base64.b64encode(validate_code_buffer).decode('utf-8')
    res = await base64_api(img=img_base64, typeid=1003)
    await validate_code_input.click()
    await validate_code_input.fill(res)
    return True


async def handle_standard_captcha(page, validate_code, img):
    res = await base64_api(img=img)
    if '|' in res:
        await click_captcha_positions(validate_code, res)
        await validate_code.locator('.geetest_submit').click()
        await validate_code.wait_for(state='hidden')
        logger.debug('验证码点击成功')
        return True
    else:
        logger.error(res)
        return False


async def click_captcha_positions(validate_code, positions_str):
    for part in positions_str.split('|'):
        x, y = part.split(',')
        await validate_code.click(position={"x": int(x), "y": int(y)})
        await asyncio.sleep(.5)


async def close_resources(page, context, browser):
    try:
        await page.close()
        await context.close()
        await browser.close()
    except Exception as e:
        logger.error(f'Error closing browser resources: {e}', exc_info=True)


async def handle_login_failure(account: Account):
    send_message(account.user.bot_token, account.user.group_id,
                 f'{account.url}:加载超时，请检查是否后台更换了链接')
    return


async def base64_api(uname='luffy230505', pwd='qwer12345', img='', typeid=20):
    logger.info('Calling third part interfaces')
    data = {
        "username": uname,
        "password": pwd,
        "typeid": typeid,
        "image": img,
        'softid': '8d13df0efe074035b54ee9c2bef85106'
    }
    async with aiohttp.ClientSession() as session:
        async with session.post("http://api.ttshitu.com/predict", json=data) as response:
            result = await response.json()
            if result['success']:
                return result["data"]["result"]
            else:
                return result["message"]


def login(account: Account):
    headers = asyncio.run(playwright_login(account))
    if headers:
        account.headers = headers
        persistence(account, headers)
        return account
    else:
        send_message(account.user.bot_token, account.user.group_id,
                     f'{account.url}:加载超时，请检查是否后台更换了链接')
    return


def persistence(account: Account, headers: dict):
    logger.info(f'Persisting headers for account {account.name}')
    with db.Session() as session:
        db_account = session.query(Account).filter(Account.username == account.username,
                                                   Account.password == account.password).one()
        db_account.headers = headers
        session.commit()
        logger.info(f'Headers persisted for account {account.name}')
